'' *****************************
'' *  Timer                    *
'' *  (C) 2006 Parallax, Inc.  *
'' *****************************
''
'' written by: Jon Williams
''             jwilliams@parallax.com
''
'' updated:    16 APR 2006



CON

  TIX_DLY = 1_000_000 / 1_000                           ' for 1 ms tix period
  MAX_STR = 16                                          ' maximum string length

  #0, TMR_TIX, TMR_SECS, TMR_MINS, TMR_HRS, TMR_DAYS

  
VAR

  long  cogon, cog                                      ' cog status
  long  tmrStack[12]                                    ' local stack

  long  tix, scs, mns, hrs, dys                         ' timer registers
  long  running                                         ' timer status
  long  updating                                        ' updating flag
  byte  tmrStr[MAX_STR + 1]                             ' add one for zero terminator


PUB start : okay

'' Load background timer if cog available
'' -- timer is on hold and registers cleared

  stop                                                  ' stop cog for this object if running
  reset
  okay := cogon := (cog := cognew(updateTimer, @tmrStack)) > 0


PUB stop

'' Unload timer object - frees a cog

  if cogon~                                             ' if object running, mark stopped
    cogstop(cog)                                        ' stop the cog


PUB set(hr, mn, sc)

'' Put timer on hold and set hours, minutes, and seconds
'' -- tix and days registers are cleared
'' -- hrs, mins, and secs are forced positive and truncated if required

  reset                                                 ' clear all, hold timer
  hrs := ||hr // 24                                     ' update registers
  mns := ||mn // 60
  scs := ||sc // 60


PUB reset

'' Put timer on hold and clear registers

  waitForUpdate                                         ' let current update complete
  hold                                                  ' stop timer
  longfill(@tix, 0, 5)                                  ' clear registers


PUB hold

'' Put timer on hold without clearing registers

  running~                                              ' disable timer (set to false)


PUB run

'' Allow timer to run and update registers

  running~~                                             ' enable timer (set to true)


PUB rdReg(reg) 

'' Read a timer register

  if lookdown(reg : TMR_TIX..TMR_DAYS)                  ' valid register?
    waitforUpdate

    return tix[reg]                                     ' return requested register 


PUB wrReg(reg, value)

'' Write value to timer register
'' -- value is forced positive and truncated if required for register

  if lookdown(reg : TMR_TIX..TMR_DAYS)                  ' valid register?
    waitForUpdate  
    tix[reg] := ||value // lookupz(reg : 1000, 60, 60, 24, posx)   
  

PUB snapShot(destAddr)

'' Copies timer regs (5 longs) to destAddr
'' -- makes transfer immediately after current update

  waitForUpdate    
  longmove(destAddr, @tix, 5)                           ' move timer registers to destination
  

PUB showTimer | tx, sc, mn, hr, dy

'' Returns address of string in format HH:MM:SS.TTT

  snapShot(@tx)                                         ' capture registers after current update
  bytefill(@tmrStr, 0, MAX_STR + 1)                     ' clear string   

  tmrStr[0]  := (hr / 10) + "0"                         ' convert 10's digit to ASCII char
  tmrStr[1]  := (hr // 10) + "0"                        ' convert 1's digit to ASCII char
  tmrStr[2]  := ":"
  tmrStr[3]  := (mn / 10) + "0"
  tmrStr[4]  := (mn // 10) + "0"
  tmrStr[5]  := ":"    
  tmrStr[6]  := (sc / 10) + "0"
  tmrStr[7]  := (sc // 10) + "0"
  tmrStr[8]  := "."    
  tmrStr[9]  := (tx / 100) + "0"                        ' 1/10ths digit
  tmrStr[10] := (tx // 100 / 10) + "0"                  ' 1/100ths digit
  tmrStr[11] := (tx // 10) + "0"                        ' 1/1000ths digit

  return @tmrStr                                        ' return address of string
  

PRI waitForUpdate

'' Hold program until current register update cycle is complete
'' -- prevents data synchronization issues

  if running                                            ' live updates in progress?
    updating~~                                          '   yes set updating flag
    repeat while updating                               '   wait for flag to clear

  
PRI updateTimer | c

'' Updates timer registers
'' -- timer.start method launches this into separate cog

  c := cnt                                              ' sync with system counter
  repeat                                                ' run until cog unloaded
    waitcnt(c += clkfreq / TIX_DLY)                     ' wait 1 ms
    if running                                          ' if timer enabled
      tix := ++tix // 1000                              '   update tix
      if (tix == 0)                                     '   rollover?
        scs := ++scs // 60                              '     yes, update seconds
        if (scs == 0)                                   '     rollover?
          mns := ++mns // 60                            '       yes, update minutes
          if (mns == 0)                                 '       rollover?
            hrs := ++hrs // 24                          '         yes, update hours
            if (hrs == 0)                               '         rollover? 
              ++dys                                     '           yes, increment days
              
    updating~                                           ' clear updating flag

      